#include "ExampleApplication.h"

class TutorialFrameListener : public ExampleFrameListener
{
 public:
 TutorialFrameListener(RenderWindow* win, Camera* cam, SceneManager *sceneMgr)
   : ExampleFrameListener(win, cam, false, false)
    {
      // key and mouse state tracking
      mMouseDown = false;
      mToggle = 0.0;
      
      // Populate the camera and scene manager containers
      mCamNode = cam->getParentSceneNode();
      mSceneMgr = sceneMgr;
      
      // set the rotation and move speed
      mRotate = 0.13;
      mMove = 10;
    }

  // Overriding the default processUnbufferedKeyInput so the key updates we define
  // later on work as intended.
  bool processUnbufferedKeyInput(const FrameEvent& evt)
  {
    return true;
  }

  // Overriding the default processUnbufferedMouseInput so the Mouse updates we define
  // later on work as intended. 
  bool processUnbufferedMouseInput(const FrameEvent& evt)
  {
    return true;
  }

  bool frameStarted(const FrameEvent &evt)
  {
    mMouse->capture();
    mKeyboard->capture();

    bool currMouse = mMouse->getMouseState().buttonDown(OIS::MB_Left);

    if (currMouse && ! mMouseDown)
      {
	Light *light = mSceneMgr->getLight("Light1");
	light->setVisible(! light->isVisible());
      } // if

    mMouseDown = currMouse;
    mToggle -= evt.timeSinceLastFrame;

    if ((mToggle < 0.0f ) && mKeyboard->isKeyDown(OIS::KC_1))
      {
	mToggle = 0.5f;
	mCamera->getParentSceneNode()->detachObject(mCamera);
	mCamNode = mSceneMgr->getSceneNode("CamNode1");
	mCamNode->attachObject(mCamera);
      }

    else if ((mToggle < 0.0f) && mKeyboard->isKeyDown(OIS::KC_2))
      {
	mToggle = 0.5f;
	mCamera->getParentSceneNode()->detachObject(mCamera);
	mCamNode = mSceneMgr->getSceneNode("CamNode2");
	mCamNode->attachObject(mCamera);
      }

    if(mKeyboard->isKeyDown(OIS::KC_ESCAPE))
      return false;

    Vector3 transVector = Vector3::ZERO;

    if (mKeyboard->isKeyDown(OIS::KC_UP) || mKeyboard->isKeyDown(OIS::KC_W))
      transVector.z += mMove;
    if (mKeyboard->isKeyDown(OIS::KC_DOWN) || mKeyboard->isKeyDown(OIS::KC_S))
      transVector.z -= mMove;

    if (mKeyboard->isKeyDown(OIS::KC_LEFT) || mKeyboard->isKeyDown(OIS::KC_Q))
      transVector.x += mMove;
    if (mKeyboard->isKeyDown(OIS::KC_RIGHT) || mKeyboard->isKeyDown(OIS::KC_E))
      transVector.x -= mMove;

    if (mKeyboard->isKeyDown(OIS::KC_SPACE))
      transVector.y += mMove;
    if (mKeyboard->isKeyDown(OIS::KC_X))
      transVector.y -= mMove;

    
    //if (mKeyboard->isKeyDown(OIS::KC_X) || mKeyboard->isKeyDown(OIS::KC_E))
      //node->pitch(Degree(-mRotate), Node::TS_LOCAL);
    //mCamNode->translate(transVector * evt.timeSinceLastFrame, Node::TS_LOCAL);
    
    SceneNode *node = mSceneMgr->getSceneNode("ZeldaNode");
    node->translate(transVector * evt.timeSinceLastFrame, Node::TS_LOCAL);

    if (mKeyboard->isKeyDown(OIS::KC_D))
      node->yaw(Degree(-mRotate*10), Node::TS_WORLD);
    if (mKeyboard->isKeyDown(OIS::KC_A))
      node->yaw(Degree(mRotate*10), Node::TS_WORLD);

    if (mMouse->getMouseState().buttonDown(OIS::MB_Right))
      {
	mCamNode->yaw(Degree(-mRotate * mMouse->getMouseState().X.rel), Node::TS_WORLD);
	mCamNode->pitch(Degree(-mRotate * mMouse->getMouseState().Y.rel), Node::TS_LOCAL);
      }

    /*    if (mKeyboard->isKeyDown(OIS::KC_8)) {
      Vector3 transVector = Vector3::ZERO;
      transVector.x += mMove*.01;
      }*/
    //Vector3 pos = node->getPosition();
    //mCamNode->setPosition(pos);

    //node->attachObject(mCamNode);

    return true;

  }
 protected:
  bool mMouseDown;       // Whether or not the left mouse button was down last frame
  Real mToggle;          // The time left until next toggle
  Real mRotate;          // The rotate constant
  Real mMove;            // The movement constant
  SceneManager *mSceneMgr;   // The current SceneManager
  SceneNode *mCamNode;   // The SceneNode the camera is currently attached to
};
